@@ -8,25 +8,36 @@ from tqdm import tqdm
 import os
 import copy
 
-def filter_table(table, samples_pattern, max_occurence = None):
+#pylint: disable=too-many-arguments
+def filter_table(table, use_samples_pattern = False, samples_pattern = '', max_occurence = None, min_rel_occurence = 0, max_rel_occurence = 1):
     
-    table_matched = table[table.index.str.contains(samples_pattern, case = False)]
-    matched_samples = list(table_matched.index)
-    table_matched = table_matched.loc[:, (table_matched != 0).any(axis=0)]
-    count_null = table_matched.replace(0, np.nan).isnull().sum()
-    
-    if max_occurence is not None:
-        excluded_features = count_null[count_null < (len(table_matched)-max_occurence)].index
-        table_filtered = table.drop(excluded_features, axis=1)
-    else: 
-        table_filtered = table
-    
-    table_filtered = table_filtered.drop(matched_samples, axis=0)
-    table_filtered = table_filtered.astype(float)
-    table_filtered = table_filtered.loc[:, (table_filtered != 0).any(axis=0)]
+    if use_samples_pattern:
+        table_matched = table[table.index.str.contains(samples_pattern, case = False)]
+        matched_samples = list(table_matched.index)
+        table_matched = table_matched.loc[:, (table_matched != 0).any(axis=0)]
+        count_null = table_matched.replace(0, np.nan).isnull().sum()
+        
+        if max_occurence is not None:
+            excluded_features = count_null[count_null < (len(table_matched)-max_occurence)].index
+            table_filtered = table.drop(excluded_features, axis=1)
+        else: 
+            table_filtered = table
+        
+        table_filtered = table_filtered.drop(matched_samples, axis=0)
+        table_filtered = table_filtered.astype(float)
+        table_filtered = table_filtered.loc[:, (table_filtered != 0).any(axis=0)]    
+    else:
+        table_filtered = table.replace(0, np.nan)
+        len_table = len(table_filtered)
+        min_rel = min_rel_occurence*len_table
+        max_rel = max_rel_occurence*len_table
+        table_filtered = table_filtered.loc[:, table_filtered.count(axis=0) <= max_rel]
+        table_filtered = table_filtered.loc[:, table_filtered.count(axis=0) >= min_rel]
+        table_filtered = table_filtered.fillna(0).astype(float)
     
     return table_filtered
-    
+        
+           
 @dataclass
 class SpectraDocuments:
     """Create a SpectraDocuments dataclass object containing spectra documents and metadata
@@ -101,19 +112,22 @@ class FeatureTable:
         else:
             raise ValueError("software argument missing, choose one of the currently supported pre-processing softwares: [mzmine, xcms, msdial]")
         
-    def filter(self, samples_pattern, max_occurence = None):
+    def filter(self, use_samples_pattern = False, samples_pattern = '', max_occurence = None, min_rel_occurence = 0, max_rel_occurence = 1):
         """Filter a feature table: remove samples matching samples_pattern
         AND remove features occuring in more than n = max_occurence samples matched by samples_pattern
 
         Args:
+            use_samples_pattern (bool): filter using a str pattern
             samples_pattern (str): the str pattern to match in samples to filter
-            max_occurence (int): maximal number of occurence allowed in matched samples before removing a feature/word
+            max_occurence (int): maximal number of occurence allowed in matched samples before removing a feature
+            min_rel_occurence (float): remove features contained in less than (min_rel_occurence * 100) percent of the samples
+            max_rel_occurence (float): remove features contained in more than (max_rel_occurence * 100) percent of the samples
 
         Returns:
             self.filtered_feature_table (DataFrame): A filtered feature table
         """
         output = copy.deepcopy(self)
-        output.feature_table = filter_table(output.feature_table, samples_pattern, max_occurence)            
+        output.feature_table = filter_table(output.feature_table, use_samples_pattern, samples_pattern, max_occurence, min_rel_occurence, max_rel_occurence)            
         return output
     
     def export_matrix(self, path, sep = ','):
@@ -169,12 +183,13 @@ class MemoMatrix:
         memo_matrix.index.name = 'filename'
         self.memo_matrix = memo_matrix
 
-    def memo_from_unaligned_samples(self, path_to_samples_dir, min_relative_intensity = 0.01,
+    def memo_from_unaligned_samples(self, path_to_samples_dir, pattern_to_match = '.mgf', min_relative_intensity = 0.01,
     max_relative_intensity = 1.00, min_peaks_required = 10, losses_from = 10, losses_to = 200, n_decimals = 2):
         """Generate a Memo matrix from a list of individual .mgf files
 
         Args:
-            path_to_samples_dir (str): Path to the directory where individual .mgf files are gathered
+            path_to_samples_dir (str): Path to the directory where individual .mgf files are gathered. Subfolders will also be checked.
+            pattern_to_match (str): Shared pattern between all spectra files to input. Will be removed in memo_matrix.index.
             min_relative_intensity (float): Minimal relative intensity to keep a peak
             max_relative_intensity (float): Maximal relative intensity to keep a peak
             min_peaks_required (int): Minimum number of peaks to keep a spectrum
@@ -185,15 +200,22 @@ class MemoMatrix:
         Returns:
             self.memo_matrix (DataFrame): A MEMO matrix
         """
-        #pylint: disable=too-many-arguments
+        #pylint: disable=too-many-locals
+                  
         dic_memo = {}
         mgf_file = []
-        for file in os.listdir(path_to_samples_dir):
-            if file.endswith(".mgf"):
-                mgf_file.append(file)
-        for file in tqdm(mgf_file):
+        path_to_mg_file = []
+        
+        for (root, _, files) in os.walk(path_to_samples_dir, topdown=True):
+            for file in files:                
+                if file.endswith(pattern_to_match):
+                    path_to_match_file = os.path.join(root, file)
+                    path_to_mg_file.append(path_to_match_file)
+                    mgf_file.append(file)
+
+        for path, file in tqdm(zip(path_to_mg_file, mgf_file), total=len(path_to_mg_file)):
             spectra = import_data.load_and_filter_from_mgf(
-                path = os.path.join(path_to_samples_dir, file), min_relative_intensity = min_relative_intensity,
+                path = path, min_relative_intensity = min_relative_intensity,
                 max_relative_intensity = max_relative_intensity, loss_mz_from = losses_from, loss_mz_to = losses_to, n_required = min_peaks_required
                 )
 
@@ -201,23 +223,26 @@ class MemoMatrix:
             documents = list(doc.words for doc in documents)
             documents = [item for sublist in documents for item in sublist]
             documents = dict(Counter(documents))
-            dic_memo[file.replace('.mgf', '')] = documents
+            dic_memo[file.replace(pattern_to_match, '')] = documents
 
         self.memo_matrix = pd.DataFrame.from_dict(dic_memo, orient='index').fillna(0)
 
-    def filter(self, samples_pattern, max_occurence = None):
+    def filter(self, use_samples_pattern = False, samples_pattern = '', max_occurence = None, min_rel_occurence = 0, max_rel_occurence = 1):
         """Filter a MEMO matrix: remove samples matching samples_pattern
         AND remove features occuring in more than n = max_occurence samples matched by samples_pattern
 
         Args:
+            use_samples_pattern (bool): filter using a str pattern
             samples_pattern (str): the str pattern to match in samples to filter
-            max_occurence (int): maximal number of occurence allowed in matched samples before removing a feature/word
+            max_occurence (int): maximal number of occurence allowed in matched samples before removing a word
+            min_rel_occurence (float): remove words contained in less than (min_rel_occurence * 100 percent) of the samples
+            max_rel_occurence (float): remove words contained in more than (max_rel_occurence * 100 percent) of the samples
 
         Returns:
             self.memo_matrix (DataFrame): A filtered feature table matrix
         """
         output = copy.deepcopy(self)
-        output.memo_matrix = filter_table(output.memo_matrix, samples_pattern, max_occurence)        
+        output.memo_matrix = filter_table(output.memo_matrix, use_samples_pattern, samples_pattern, max_occurence, min_rel_occurence, max_rel_occurence)        
         return output
 
     def merge_memo(self, memomatrix_2, drop_not_in_common=False):
